@@ -1,3 +1,18 @@
+//! # Silver-Platter
+//!
+//! Silver-Platter makes it possible to contribute automatable changes to source
+//! code in a version control system
+//! ([codemods](https://github.com/jelmer/awesome-codemods)).
+//!
+//! It automatically creates a local checkout of a remote repository,
+//! makes user-specified changes, publishes those changes on the remote hosting
+//! site and then creates a pull request.
+//!
+//! In addition to that, it can also perform basic maintenance on branches
+//! that have been proposed for merging - such as restarting them if they
+//! have conflicts due to upstream changes.
+
+#![deny(missing_docs)]
 pub mod batch;
 pub mod candidates;
 pub mod checks;
@@ -12,7 +27,7 @@ pub mod run;
 pub mod utils;
 pub mod vcs;
 pub mod workspace;
-pub use breezyshim::branch::{Branch, RegularBranch};
+pub use breezyshim::branch::{Branch, GenericBranch};
 pub use breezyshim::controldir::{ControlDir, Prober};
 pub use breezyshim::forge::{Forge, MergeProposal};
 pub use breezyshim::transport::Transport;
@@ -22,21 +37,27 @@ use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use std::path::Path;
 
 #[derive(Debug, Serialize, Deserialize, Clone, Copy, PartialEq, Eq, Default)]
+/// Publish mode
 pub enum Mode {
     #[serde(rename = "push")]
+    /// Push to the target branch
     Push,
 
     #[serde(rename = "propose")]
+    /// Propose a merge
     Propose,
 
     #[serde(rename = "attempt-push")]
     #[default]
+    /// Attempt to push to the target branch, falling back to propose if necessary
     AttemptPush,
 
     #[serde(rename = "push-derived")]
+    /// Push to a branch derived from the script name
     PushDerived,
 
     #[serde(rename = "bts")]
+    /// Bug tracking system
     Bts,
 }
 
@@ -93,16 +114,24 @@ impl pyo3::ToPyObject for Mode {
     }
 }
 
+/// Returns the branch name derived from a script name
 pub fn derived_branch_name(script: &str) -> &str {
     let first_word = script.split(' ').next().unwrap_or("");
     let script_name = Path::new(first_word).file_stem().unwrap_or_default();
     script_name.to_str().unwrap_or("")
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+/// Policy on whether to commit pending changes
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Default)]
 pub enum CommitPending {
+    /// Automatically determine pending changes
+    #[default]
     Auto,
+
+    /// Commit pending changes
     Yes,
+
+    /// Don't commit pending changes
     No,
 }
 
@@ -146,20 +175,35 @@ impl<'de> Deserialize<'de> for CommitPending {
     }
 }
 
+impl CommitPending {
+    /// Returns whether the policy is to commit pending changes
+    pub fn is_default(&self) -> bool {
+        *self == CommitPending::Auto
+    }
+}
+
+/// The result of a codemod
 pub trait CodemodResult {
+    /// Context
     fn context(&self) -> serde_json::Value;
 
+    /// Returns the value of the result
     fn value(&self) -> Option<u32>;
 
+    /// Returns the URL of the target branch
     fn target_branch_url(&self) -> Option<url::Url>;
 
+    /// Returns the description of the result
     fn description(&self) -> Option<String>;
 
+    /// Returns the tags of the result
     fn tags(&self) -> Vec<(String, Option<RevisionId>)>;
 
+    /// Returns the context as a Tera context
     fn tera_context(&self) -> tera::Context {
         tera::Context::from_value(self.context()).unwrap()
     }
 }
 
+/// The version of the library
 pub const VERSION: &str = env!("CARGO_PKG_VERSION");
